// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <algorithm>	 // for copy, copy_backward
#include <functional>	 // for function

#include "CallerEnvironment.hpp"
#include "Debug.hpp"	// for LogSubsystem, LogSubsystem::SCHED, Logger
#include "Emper.hpp"
#include "Fiber.hpp"			 // for Fiber
#include "emper-common.h"	 // for workeraffinity_t
#include "lib/adt/LockedUnboundedQueue.hpp"

class Runtime;

class Scheduler : public Logger<LogSubsystem::SCHED> {
 private:
	lib::adt::LockedUnboundedQueue<Fiber> scheduleAnywhereQueue;

	void wakeupSleepingWorkersIfSleepingObserved();

	void wakeupSleepingWorkersAlways();

 protected:
	Runtime& runtime;
	Scheduler(Runtime& runtime);

	void addNewWorkerHook(const std::function<void(void)>& hook);

	static inline auto getAffinityBuffer(Fiber& fiber) -> workeraffinity_t* {
		return fiber.getAffinityBuffer();
	}

	static inline void increaseRefCount(Fiber& fiber) { fiber.doAtomicIncrRefCount(); }

	template <CallerEnvironment callerEnvironment = CallerEnvironment::EMPER>
	inline void onNewWork() {
		if constexpr (emper::WORKER_SLEEP) {
			if constexpr (callerEnvironment == CallerEnvironment::EMPER) {
				wakeupSleepingWorkersIfSleepingObserved();
			} else {
				wakeupSleepingWorkersAlways();
			}
		}
	}

	auto dequeFiberFromAnywhereQueue() -> Fiber* { return scheduleAnywhereQueue.dequeue(); }

 public:
	virtual void schedule(Fiber& fiber) = 0;

	virtual auto nextFiber() -> Fiber* = 0;

	void scheduleFromAnywhere(Fiber& fiber) {
		scheduleAnywhereQueue.enqueue(&fiber);

		onNewWork<CallerEnvironment::ANYWHERE>();
	}
};
