// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <deque>
#include <mutex>

#include "Common.hpp"

namespace adt {

template <typename I, const uintptr_t SIZE>
class LockedQueue {
 private:
	std::mutex queue_mutex;

	std::deque<I> deque;

 public:
	// We can not mark this method 'const' as std::lock_guard modifies
	// the mutex.
	auto isFull() -> bool {
		std::lock_guard<std::mutex> lock(queue_mutex);
		return deque.size() == SIZE;
	}

	auto pushBottom(const I item) -> bool {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.size() == SIZE) return false;

		deque.push_back(item);

		return true;
	}

	auto popTop(I* itemPtr) -> bool {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.empty()) return false;

		*itemPtr = deque.front();

		deque.pop_front();

		return true;
	}

	auto popBottom(I* itemPtr) -> bool {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.empty()) return false;

		*itemPtr = deque.back();

		deque.pop_back();

		return true;
	}
};
}	 // namespace adt
