// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <condition_variable>
#include <mutex>

namespace emper::lib::sync {

class Semaphore {
 private:
	std::mutex m;
	std::condition_variable c;
	unsigned int counter;

 public:
	Semaphore(unsigned int counter = 0) : counter(counter) {}

	inline void notify() {
		std::unique_lock<std::mutex> lock(m);
		++counter;
		c.notify_one();
	}

	inline void notify_many(unsigned int count) {
		std::unique_lock<std::mutex> lock(m);
		counter += count;
		c.notify_all();
	}

	inline void wait() {
		std::unique_lock<std::mutex> lock(m);
		c.wait(lock, [this]() { return counter > 0; });
		counter--;
	}
};

}	 // namespace emper::lib::sync
