// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include "LawsStrategyStats.hpp"

#include <atomic>
#include <iostream>

#include "LawsStrategy.hpp"

LawsStrategyStats::LawsStrategyStats(LawsStrategy& lawsStrategy)
		: scheduledFibersToRemotePriority(lawsStrategy.scheduledFibersToRemotePriority),
			scheduledFibersToLocal(lawsStrategy.scheduledFibersToLocal),
			dispatchedFiberFromPriority(lawsStrategy.dispatchedFiberFromPriority),
			dispatchedFiberFromLocal(lawsStrategy.dispatchedFiberFromLocal),
			dispatchedFiberStolen(lawsStrategy.dispatchedFiberStolen),
			dispatchedFiberFromAnywhere(lawsStrategy.dispatchedFiberFromAnywhere) {}

auto LawsStrategyStats::getScheduledFibersToRemotePriority() const -> uint64_t {
	return scheduledFibersToRemotePriority;
}

auto LawsStrategyStats::getScheduledFibersToLocal() const -> uint64_t {
	return scheduledFibersToLocal;
}

auto LawsStrategyStats::getDispatchedFiberFromPriority() const -> uint64_t {
	return dispatchedFiberFromPriority;
}

auto LawsStrategyStats::getDispatchedFiberFromLocal() const -> uint64_t {
	return dispatchedFiberFromLocal;
}

auto LawsStrategyStats::getDispatchedFiberStolen() const -> uint64_t {
	return dispatchedFiberStolen;
}

auto LawsStrategyStats::getDispatchedFiberFromAnywhere() const -> uint64_t {
	return dispatchedFiberFromAnywhere;
}

void LawsStrategyStats::print() {
	std::cout << "LawsStrategyStats"
						<< " scheduledFibersToRemotePriority:" << scheduledFibersToRemotePriority
						<< " scheduledFibersToLocal:" << scheduledFibersToLocal
						<< " dispatchedFiberFromPriority:" << dispatchedFiberFromPriority
						<< " dispatchedFiberFromLocal:" << dispatchedFiberFromLocal
						<< " dispatchedFiberStolen:" << dispatchedFiberStolen
						<< " dispatchedFiberFromAnywhere:" << dispatchedFiberFromAnywhere << std::endl;
}
