// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <cstddef>	// for size_t

#include "Scheduler.hpp"					// for Scheduler
#include "emper-common.h"					// for ATTR_UNUSED
#include "lib/adt/WsClQueue.hpp"	// for WsClQueue

class Fiber;
class Runtime;
class WsStrategy;

class WsScheduler : public Scheduler {
	template <size_t SIZE>
#ifdef EMPER_LOCKED_WS_QUEUE
	using WsQueue = adt::LockedQueue<Fiber*, SIZE>;
#else
	using WsQueue = adt::WsClQueue<Fiber*, SIZE>;
#endif

 public:
	static const int QUEUE_SIZE = 1024;

 private:
	WsQueue<QUEUE_SIZE>** queues;
	static thread_local WsQueue<QUEUE_SIZE> queue;

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wattributes"
	WsStrategy& wsStrategy
#ifndef EMPER_STATS
			ATTR_UNUSED
#endif
			;
#pragma GCC diagnostic pop

 public:
	WsScheduler(Runtime& runtime, WsStrategy& wsStrategy);

	void schedule(Fiber& fiber) override;

	auto nextFiber() -> Fiber* override;
};
