// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include <cstdlib>	// for exit, EXIT_FAILURE, EXIT_SUCCESS
#include <thread>

#include "BinaryPrivateSemaphore.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"	// for Runtime
#include "emper-common.h"
#include "emper.hpp"

static void testSignalPrivateSemaphoreFromAnywhere() {
	BinaryPrivateSemaphore bpsTest;
	BinaryPrivateSemaphore bpsSignalledFromAnywhere;

	Fiber* fiberWaitingForBps = Fiber::from([&] {
		bpsSignalledFromAnywhere.wait();

		bpsTest.signal();
	});

	async(fiberWaitingForBps);

	// TODO: Use std::jthread once EMPER uses C++20.
	std::thread signallingThread([&] { bpsSignalledFromAnywhere.signalFromAnywhere(); });

	bpsTest.wait();

	// TODO: Remove this once we use std::jthread when EMPER uses C++20.
	signallingThread.join();

	exit(EXIT_SUCCESS);
}

auto main(UNUSED_ARG int argc, UNUSED_ARG char* argv[]) -> int {
	Runtime runtime;

	Fiber* alphaFiber = Fiber::from(&testSignalPrivateSemaphoreFromAnywhere);

	runtime.scheduleFromAnywhere(*alphaFiber);

	runtime.waitUntilFinished();

	return EXIT_FAILURE;
}
