// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#include <algorithm>	 // for copy, copy_backward
#include <cstdlib>		 // for exit, EXIT_FAILURE, EXIT_SUCCESS
#include <functional>	 // for bind
#include <thread>

#include "Actor.hpp"
#include "BinaryPrivateSemaphore.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"	// for Runtime
#include "emper-common.h"

class SignallingActor : public Actor<unsigned int> {
 private:
	BinaryPrivateSemaphore& bps;

 protected:
	void receive(UNUSED_ARG unsigned int signalCount) override { bps.signal(); }

 public:
	SignallingActor(BinaryPrivateSemaphore& bps) : bps(bps){};
};

static void testTellActorFromAnywhere() {
	BinaryPrivateSemaphore bps;
	SignallingActor signallingActor(bps);
	signallingActor.start();

	// TODO: Use std::jthread once EMPER uses C++20.
	std::thread signallingThread([&] { signallingActor.tellFromAnywhere(1); });

	bps.wait();

	// TODO: Remove this once we use std::jthread when EMPER uses C++20.
	signallingThread.join();

	exit(EXIT_SUCCESS);
}

auto main(UNUSED_ARG int argc, UNUSED_ARG char* argv[]) -> int {
	Runtime runtime;

	Fiber* alphaFiber = Fiber::from(&testTellActorFromAnywhere);

	runtime.scheduleFromAnywhere(*alphaFiber);

	runtime.waitUntilFinished();

	return EXIT_FAILURE;
}
